#!/usr/bin/perl
#
# Sensor for gathering hardware data for Zabbix via
# IPMI.
# Copyright (c) 2008 Jakov Sosic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Changes and Modifications
# * Thu May 15 03:40:39 CEST 2008       
#       - script created
#

# zabbix path
my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $ipmitool="/usr/bin/ipmitool";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd";

# signal handling
$SIG{'ALRM'} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
};

local $SIG{TERM} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
};

# fetch status from mod_status
alarm 20;
my $cmd = "sudo $ipmitool sdr list";
my $server_status = qx($cmd);

# get information from status output
alarm 3;
my($ChassisTemp, $CPU1temp,$CPU2temp,$CPU3temp,$CPU4temp, $Fan1,$Fan2,$Fan3,$Fan4, $Fan5, $Fan6);
$ChassisTemp = $1 if ($server_status =~ /Ambient Temp \s+\|\s+([\d]+)\s+degrees/);
$CPU1temp = $1 if ($server_status =~ s/Temp \s+\|\s+([\d]+)\s+degrees//);
$CPU2temp = $1 if ($server_status =~ s/Temp \s+\|\s+([\d]+)\s+degrees//);
$CPU3temp = $1 if ($server_status =~ s/Temp \s+\|\s+([\d]+)\s+degrees//);
$CPU4temp = $1 if ($server_status =~ s/Temp \s+\|\s+([\d]+)\s+degrees//);
$Fan1 = $1 if ($server_status =~ /FAN 1 .+\|\s+([\d]+)\s+RPM/);
$Fan2 = $1 if ($server_status =~ /FAN 2 .+\|\s+([\d]+)\s+RPM/);
$Fan3 = $1 if ($server_status =~ /FAN 3 .+\|\s+([\d]+)\s+RPM/);
$Fan4 = $1 if ($server_status =~ /FAN 4 .+\|\s+([\d]+)\s+RPM/);
$Fan5 = $1 if ($server_status =~ /FAN 5 .+\|\s+([\d]+)\s+RPM/);
$Fan6 = $1 if ($server_status =~ /FAN 6 .+\|\s+([\d]+)\s+RPM/);
#print $ChassisTemp."\n";


# send information to Zabbix server
alarm 10;
if ( qx($zabbix_send_command -k ipmi.temp.cpu1 -o $CPU1temp) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.cpu2 -o $CPU2temp) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.cpu3 -o $CPU3temp) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.cpu4 -o $CPU4temp) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.temp.chassis -o $ChassisTemp) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.1 -o $Fan1) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.2 -o $Fan2) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.3 -o $Fan3) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.4 -o $Fan4) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.5 -o $Fan5) =~ /failed:\ 1/ ) { print"0\n"; exit(-1);}
if ( qx($zabbix_send_command -k ipmi.fan.6 -o $Fan6) =~ /failed:\ 1/) { print"0\n"; exit(-1);}
print "1\n";
exit(0);
# EOF

